function toggleMode(toggle, id) {
  gantt.$zoomToFit = !gantt.$zoomToFit;
  if (gantt.$zoomToFit) {
    toggle.innerHTML = "Set default Scale";
    //Saving previous scale state for future restore
    saveConfig();
    zoomToFit(id);
  } else {

    toggle.innerHTML = "Zoom to Fit";
    //Restore previous scale state
    restoreConfig();
    gantt.render();
  }
}

var cachedSettings = {};

function saveConfig() {
  var config = gantt.config;
  cachedSettings = {};
  cachedSettings.scales = config.scales;
  cachedSettings.start_date = config.start_date;
  cachedSettings.end_date = config.end_date;
  cachedSettings.scroll_position = gantt.getScrollState();
}

function restoreConfig() {
  applyConfig(cachedSettings);
}

function applyConfig(config, dates, id = 0) {
  if (Gantt[id] !== undefined) {
    Gantt[id].config.scales = config.scales;
  } else {
    gantt.config.scales = config.scales;
  }

  // restore the previous scroll position
  if (config.scroll_position) {
    setTimeout(function () {
      gantt.scrollTo(config.scroll_position.x, config.scroll_position.y)
    }, 4)
  }
}


function zoomToFit(id = 0) {
  let project = gantt.getSubtaskDates(),
    areaWidth = 300;
  if (Gantt !== undefined && Gantt[id] !== undefined) {
    project = Gantt[id].getSubtaskDates(),
      areaWidth = Gantt[id].$task.offsetWidth;
  } else {
    areaWidth = gantt.$task.offsetWidth;
  }
  let scaleConfigs = zoomConfig.levels;
  for (var i = 0; i < scaleConfigs.length; i++) {
    var columnCount = getUnitsBetween(project.start_date, project.end_date, scaleConfigs[i].scales[scaleConfigs[i].scales.length - 1].unit, scaleConfigs[i].scales[0].step);
    if ((columnCount + 2) * gantt.config.min_column_width <= areaWidth) {
      break;
    }
  }
  if (i == scaleConfigs.length) {
    i--;
  }

  if (Gantt !== undefined && Gantt[id] !== undefined) {
    Gantt[id].ext.zoom.setLevel(scaleConfigs[i].name);
  } else {
    gantt.ext.zoom.setLevel(scaleConfigs[i].name);
  }
  applyConfig(scaleConfigs[i], project, id);
}

// Get number of columns in timeline.
function getUnitsBetween(from, to, unit, step) {
  var start = new Date(from),
    end = new Date(to);
  var units = 0;
  while (start.valueOf() < end.valueOf()) {
    units++;
    start = gantt.date.add(start, step, unit);
  }
  return units;
}

function zoom_in(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].ext.zoom.zoomIn();
    Gantt[id].$zoomToFit = false;
  } else {
    gantt.ext.zoom.zoomIn();
    gantt.$zoomToFit = false;
  }
  jQuery(".zoomToFit").html("<i class='bi bi-arrows-fullscreen'></i> Zoom to Fit");
}

function zoom_out(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].ext.zoom.zoomOut();
    Gantt[id].$zoomToFit = false;
  } else {
    gantt.ext.zoom.zoomOut();
    gantt.$zoomToFit = false;
  }
  jQuery(".zoomToFit").html("<i class='bi bi-fullscreen-exit'></i> Zoom to Fit");
}

function collapse(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].batchUpdate(function () {
      Gantt[id].eachTask(function (task) {
        Gantt[id].close(task.id)
      });
    });
  } else {
    gantt.batchUpdate(function () {
      gantt.eachTask(function (task) {
        gantt.close(task.id)
      });
    });
  }
}

function expand(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].batchUpdate(function () {
      Gantt[id].eachTask(function (task) {
        Gantt[id].open(task.id)
      });
    });
  } else {
    gantt.batchUpdate(function () {
      gantt.eachTask(function (task) {
        gantt.open(task.id)
      });
    });
  }
}

function undo(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].undo();
  } else {
    gantt.undo();
  }
}

function redo(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].redo();
  } else {
    gantt.redo();
  }
}

function exportToExcel(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].exportToExcel();
  } else {
    gantt.exportToExcel();
  }
}

function exportToMSProject(id, config) {
  if (typeof Gantt !== "undefined" && Gantt[id] !== "undefined") {
    Gantt[id].exportToMSProject(config);
  } else {
    gantt.exportToMSProject(config);
  }
}

function exportToICal(id) {
  if (typeof Gantt !== "undefined" && Gantt[id] !== "undefined") {
    Gantt[id].exportToICal();
  } else {
    gantt.exportToICal();
  }
}

function fullscreen(id) {
  if (typeof Gantt !== "undefined" && Gantt[id] !== "undefined") {
    Gantt[id].ext.fullscreen.toggle();
  } else {
    gantt.ext.fullscreen.toggle();
  }
}

var zoomConfig = {
  levels: [
    // Hours.
    {
      name: "hour",
      scale_height: 27,
      scales: [
        {unit: "day", step: 1, format: "%d %M"},
        {unit: "hour", step: 1, format: "%H:%i"},
      ]
    },
    // Days.
    {
      name: "day",
      scale_height: 27,
      scales: [
        {unit: "day", step: 1, format: "%d %M"}
      ]
    },
    // Weeks.
    {
      name: "week",
      scale_height: 50,
      scales: [
        {
          unit: "week", step: 1, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%d %M");
            var endDate = gantt.date.add(date, -6, "day");
            var weekNum = gantt.date.date_to_str("%W")(date);
            return "#" + weekNum + ", " + dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
        {unit: "day", step: 1, format: "%j %D"}
      ]
    },
    // Months.
    {
      name: "month",
      scale_height: 50,
      scales: [
        {unit: "month", step: 1, format: "%F, %Y"},
        {
          unit: "week", step: 1, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%d %M");
            var endDate = gantt.date.add(gantt.date.add(date, 1, "week"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        }
      ]
    },
    // Quarters.
    {
      name: "quarter",
      height: 50,
      scales: [
        {
          unit: "quarter", step: 3, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%M %y");
            var endDate = gantt.date.add(gantt.date.add(date, 3, "month"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
        {unit: "month", step: 1, format: "%M"},
      ]
    },
    // Years.
    {
      name: "year",
      scale_height: 50,
      scales: [
        {
          unit: "year", step: 5, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%Y");
            var endDate = gantt.date.add(gantt.date.add(date, 5, "year"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        }
      ]
    },
    // Decades.
    {
      name: "year",
      scale_height: 50,
      scales: [
        {
          unit: "year", step: 100, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%Y");
            var endDate = gantt.date.add(gantt.date.add(date, 100, "year"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
        {
          unit: "year", step: 10, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%Y");
            var endDate = gantt.date.add(gantt.date.add(date, 10, "year"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
      ]
    },
  ],
  element: function () {
    return gantt.$root.querySelector(".gantt_task");
  }
};
