/**
 * @file
 */

(($, Drupal, cookies) => {
  // Marker today.
  function markerToday() {
    let dateToStr = gantt.date.date_to_str(gantt.config.task_date);

    let id = gantt.addMarker({
      start_date: new Date(),
      css: "today",
      title: dateToStr(new Date())
    });
    setInterval(function () {
      let today = gantt.getMarker(id);
      if (today) {
        today.start_date = new Date();
        today.title = dateToStr(today.start_date);
        gantt.updateMarker(id);
      }
    }, 1000 * 60);
    //Make resize marker for two columns
    gantt.attachEvent("onColumnResizeStart", function (ind, column) {
      if (!column.tree || ind == 0) {
        return;
      }

      setTimeout(function () {
        let marker = document.querySelector(".gantt_grid_resize_area");
        if (!marker) {
          return;
        }
        let cols = gantt.getGridColumns();
        let delta = cols[ind - 1].width || 0;
        if (!delta) {
          return;
        }

        marker.style.boxSizing = "content-box";
        marker.style.marginLeft = -delta + "px";
        marker.style.paddingRight = delta + "px";
      }, 1);
    });
  }

  // Undo Redo.
  function undo() {
    gantt.attachEvent("onBeforeAutoSchedule", function () {
      gantt.message("Recalculating project schedule...");
      return true;
    });
    gantt.attachEvent("onAfterTaskAutoSchedule", function (task, new_date, constraint, predecessor) {
      gantt.message({
        text: "<b>" + task.text + "</b> has been rescheduled to " + gantt.templates.task_date(new_date) + " due to <b>" + predecessor.text + "</b> constraint",
        expire: 4000
      });
    });
  }

  // Expand at bottom.
  function expand() {
    gantt.attachEvent("onCollapse", function () {
      var icon = gantt.toggleIcon;
      if (icon) {
        icon.className = icon.className.replace("fullscreen-exit", "bi-arrows-fullscreen");
      }
    });
    gantt.attachEvent("onExpand", function () {
      var icon = gantt.toggleIcon;
      if (icon) {
        icon.className = icon.className.replace("bi-arrows-fullscreen", "fullscreen-exit");
      }
    });
    gantt.attachEvent("onTemplatesReady", function () {
      var toggle = document.createElement("i");
      toggle.className = "bi bi-arrows-fullscreen gantt-fullscreen";
      gantt.toggleIcon = toggle;
      gantt.$container.appendChild(toggle);
      toggle.onclick = function () {
        gantt.ext.fullscreen.toggle();
      };
    });
  }

  // User owner.
  function byId(list, id) {
    for (let i = 0; i < list.length; i++) {
      if (list[i].key == parseInt(id)) {
        return list[i].label || "";
      }
    }
    return "";
  }

  function owners(owner) {
    gantt.form_blocks["multiselect"] = {
      render: function (sns) {
        let height = (sns.height || "23") + "px";
        let html = "<div class='gantt_cal_ltext gantt_cal_chosen gantt_cal_multiselect' style='height:" + height + ";width: 100%'><select data-placeholder='…' class='chosen-select' multiple>";
        if (sns.options) {
          for (var i = 0; i < sns.options.length; i++) {
            if(sns.unassigned_value !== undefined && sns.options[i].key == sns.unassigned_value){
              continue;
            }
            html += "<option value='" + sns.options[i].key + "'>" + sns.options[i].label + "</option>";
          }
        }
        html += "</select></div>";
        return html;
      },

      set_value: function (node, value, ev, sns) {
        node.style.overflow = "visible";
        node.parentNode.style.overflow = "visible";
        node.style.display = "inline-block";
        var select = $(node.firstChild);

        if (value) {
          value = (value + "").split(",");
          select.val(value);
        }
        else {
          select.val([]);
        }

        select.chosen();
        if (sns.onchange) {
          select.change(function () {
            sns.onchange.call(this);
          })
        }
        select.trigger('chosen:updated');
        select.trigger("change");
      },

      get_value: function (node, ev) {
        var value = $(node.firstChild).val();
        return value;
      },

      focus: function (node) {
        $(node.firstChild).focus();
      }
    };

    gantt.locale.labels.column_owner =
    gantt.locale.labels.section_owner = Drupal.t("Owner");
    gantt.config.resource_property = "staff";
    gantt.serverList("staff", owner);

    gantt.attachEvent("onTaskCreated", function (task) {
      task.staff = [];
      task.calendar_id = null;
      return true;
    });
    // Check column owner is existing?
    let isColumnExist = false;
    gantt.config.columns.forEach(function (item) {
      if (item.hasOwnProperty("name")) {
        if (item.name === "owner") {
          isColumnExist = true;
        }
        if (item.name === "text") {
          item.width = 300;
        }
      }
    });
    if (!isColumnExist) {
      gantt.config.columns.unshift({
        name: "owner", width: 80, align: "center", template: (task) => {
          if (task.type == gantt.config.types.project) {
            return "";
          }
          var result = "";
          let owners = task.owner;
          if (!owners || !owners.length) {
            return "Unassigned";
          }
          if(owners.length == 1){
            return byId(gantt.serverList('staff'), task.owner[0]);
          }
          owners.forEach( (ownerId) => {
            let owner = byId(gantt.serverList('staff'), ownerId);
            if (!owner){
              return;
            }
            result += "<div class='owner-label' title='" + owner + "'>" + owner.charAt(0).toUpperCase() + "</div>";
          });
          return result;
        }, resize: true
      });

      gantt.config.lightbox.sections.push(
        {name:"owner",height:60, type:"multiselect", options:gantt.serverList("staff"), map_to:"owner"}
      );
    }

    gantt.templates.grid_row_class =
    gantt.templates.task_row_class =
    gantt.templates.task_class = function (start, end, task) {
      let css = [];
      if (task.$virtual || task.type == gantt.config.types.project) {
        css.push("summary-bar");
      }

      if (task.owner) {
        css.push("gantt_resource_task gantt_resource_" + task.owner);
      }

      return css.join(" ");
    };

    gantt.attachEvent("onDataRender", function () {
      let styleId = "dynamicGanttStyles";
      let element = document.getElementById(styleId);
      if (!element) {
        element = document.createElement("style");
        element.id = styleId;
        document.querySelector("head").appendChild(element);
      }
      let html = [];
      let resources = gantt.serverList("staff");

      resources.forEach(function (r) {
        html.push(".gantt_task_line.gantt_resource_" + r.key + "{" +
          "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
        html.push(".gantt_row.gantt_resource_" + r.key + " .gantt_cell:nth-child(1) .gantt_tree_content{" +
          "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
      });
      element.innerHTML = html.join("");
    });

  }

  // Tooltips.
  function setTooltip() {
    let formatter = gantt.ext.formatters.durationFormatter({
      enter: "day",
      store: "hour",
      format: "day",
      hoursPerDay: 24,
      hoursPerWeek: 40,
      daysPerMonth: 30,
      short: true
    });

    // setup lightbox
    gantt.locale.labels.section_baseline = Drupal.t("Planned");
    gantt.locale.labels.baseline_enable_button = Drupal.t('Set');
    gantt.locale.labels.baseline_disable_button = Drupal.t('Remove');

    gantt.config.lightbox.sections = [
      {name: "description", height: 70, map_to: "text", type: "textarea", focus: true},
      {
        name: "type", type: "typeselect", map_to: "type", filter: function (name, value) {
          return !!(value != gantt.config.types.project);
        }
      },
      {name: "time", type: "duration", map_to: "auto"},
      {name: "baseline",  button: true, type: "duration_optional", map_to: {start_date: "planned_start", end_date: "planned_end"}}
    ];
    gantt.config.lightbox.project_sections = [
      {name: "description", height: 70, map_to: "text", type: "textarea", focus: true},
      {name: "time", type: "duration", readonly: true, map_to: "auto"},
      {name: "baseline",  button: true, type: "duration_optional", map_to: {start_date: "planned_start", end_date: "planned_end"}}
    ];
    gantt.config.lightbox.milestone_sections = [
      {name: "description", height: 70, map_to: "text", type: "textarea", focus: true},
      {
        name: "type", type: "typeselect", map_to: "type", filter: function (name, value) {
          return !!(value != gantt.config.types.project);
        }
      },
      {name: "time", type: "duration", single_date: true, map_to: "auto", formatter: formatter}
    ];

  }

  function upload(file, callback, urlImport) {
    gantt.importFromMSProject({
      data: file,
      callback: function (project) {
        if (project) {
          gantt.clearAll();
          if (project.config.duration_unit) {
            gantt.config.duration_unit = project.config.duration_unit;
          }
          // mark tasks import.
          for (let i = 0; i < project.data.data.length; i++) {
            project.data.data[i].isImport = true;
          }
          gantt.parse(project.data);
          $.post(urlImport, project.data).done(function (data) {
            console.log("Data import: " + data);
          });
          if (callback) {
            callback(project);
          }
        }
      }
    });
  }

  function importMMP(ganttId, urlImport) {
    // Import microsoft project.
    let fileDnD = fileDragAndDrop();
    fileDnD.init(gantt.$container);

    function sendFile(file) {
      fileDnD.showUpload();
      upload(file, function () {
        fileDnD.hideOverlay();
      }, urlImport)
    }

    fileDnD.onDrop(sendFile);
    // Manual Upload file MPP.
    $("#mspImportBtn" + ganttId).click(function (event) {
      event.preventDefault();
      let fileInput = document.getElementById("mspFile" + ganttId);
      if (fileInput.files[0]) {
        sendFile(fileInput.files[0]);
      }
    });
  }

  function native_dialog(setting) {
    gantt.attachEvent("onTaskDblClick", function (id, e) {
      if (id == null) {
        return false;
      }
      task = gantt.getTask(id);
      if (task['type'] == 'project') {
        return false;
      }
      else {
        let ajaxSettings = {
          url: '/paragraphs-gantt/' + id + '/edit?' + setting.current_path,
          dialogType: 'modal',
          dialog: { width: '80%' },
        };
        Drupal.ajax(ajaxSettings).execute();
        return false;
      }
    });

    gantt.attachEvent("onBeforeLightbox", function (id) {
      let task = gantt.getTask(id);
      if (task == null) {
        return false;
      }
      if (task.$new) {
        if (setting.add_task !== true) {
          gantt.deleteTask(task.id);
          return false;
        }
        let add_link = setting.add_link + '&parent=' + task.parent + '&parent_field=' + setting.parent_field;
        let ajaxSettings = {
          url: add_link,
          dialogType: 'modal',
          dialog: {width: '80%'},
        };
        Drupal.ajax(ajaxSettings).execute();
        //window.location = add_link;
        gantt.deleteTask(task.id);
        return false;
      }
      return true;
    });

  }

  let plugins = {
    fullscreen: true,
    marker: true,
    critical_path: true,
    tooltip: true,
    multiselect: true,
    // undo: true,
  };
  let i18n = {
    date: {
      month_full: [Drupal.t("January"), Drupal.t("February"), Drupal.t("March"), Drupal.t("April"), Drupal.t("May"), Drupal.t("June"),
        Drupal.t("July"), Drupal.t("August"), Drupal.t("September"), Drupal.t("October"), Drupal.t("November"), Drupal.t("December")],
      month_short: [Drupal.t("Jan"), Drupal.t("Feb"), Drupal.t("Mar"), Drupal.t("Apr"), Drupal.t("May"), Drupal.t("Jun"), Drupal.t("Jul"), Drupal.t("Aug"), Drupal.t("Sep"), Drupal.t("Oct"), Drupal.t("Nov"), Drupal.t("Dec")],
      day_full: [Drupal.t("Sunday"), Drupal.t("Monday"), Drupal.t("Tuesday"), Drupal.t("Wednesday"), Drupal.t("Thursday"), Drupal.t("Friday"), Drupal.t("Saturday")],
      day_short: [Drupal.t("Sun"), Drupal.t("Mon"), Drupal.t("Tue"), Drupal.t("Wed"), Drupal.t("Thu"), Drupal.t("Fri"), Drupal.t("Sat")]
    },
    labels: {
      new_task: Drupal.t("New task"),
      icon_save: Drupal.t("Save"),
      icon_cancel: Drupal.t("Cancel"),
      icon_details: Drupal.t("Details"),
      icon_edit: Drupal.t("Edit"),
      icon_delete: Drupal.t("Delete"),
      gantt_save_btn: Drupal.t("New Label"),
      gantt_cancel_btn: Drupal.t("New Label"),
      gantt_delete_btn: Drupal.t("New Label"),
      confirm_deleting: Drupal.t("Task will be deleted permanently, are you sure?"),
      section_description: Drupal.t("Description"),
      section_time: Drupal.t("Time period"),
      section_type: Drupal.t("Type"),
      section_baseline: Drupal.t("Planned"),
      baseline_enable_button: Drupal.t('Set'),
      baseline_disable_button: Drupal.t('Remove'),
      details_btn: Drupal.t("Details"),

      /* grid columns */
      column_wbs: "WBS",
      column_text: Drupal.t("Task name"),
      column_start_date: Drupal.t("Start time"),
      column_duration: Drupal.t("Duration"),

      /* link confirmation */
      link: Drupal.t("Link"),
      confirm_link_deleting: Drupal.t("will be deleted"),
      link_start: Drupal.t(" (start)"),
      link_end: Drupal.t(" (end)"),

      type_task: Drupal.t("Task"),
      type_project: Drupal.t("Project"),
      type_milestone: Drupal.t("Milestone"),
      type_placeholder: Drupal.t("Placeholder"),

      minutes: Drupal.t("Minutes"),
      hours: Drupal.t("Hours"),
      days: Drupal.t("Days"),
      weeks: Drupal.t("Week"),
      months: Drupal.t("Months"),
      years: Drupal.t("Years"),

      /* message popup */
      message_ok: Drupal.t("OK"),
      message_cancel: Drupal.t("Cancel"),

      /* constraints */
      section_constraint: Drupal.t("Constraint"),
      constraint_type: Drupal.t("Constraint type"),
      constraint_date: Drupal.t("Constraint date"),
      asap: Drupal.t("As Soon As Possible"),
      alap: Drupal.t("As Late As Possible"),
      snet: Drupal.t("Start No Earlier Than"),
      snlt: Drupal.t("Start No Later Than"),
      fnet: Drupal.t("Finish No Earlier Than"),
      fnlt: Drupal.t("Finish No Later Than"),
      mso: Drupal.t("Must Start On"),
      mfo: Drupal.t("Must Finish On"),

      /* resource control */
      resources_filter_placeholder: Drupal.t("type to filter"),
      resources_filter_label: Drupal.t("hide empty")
    }
  };
  Drupal.behaviors.paragraphs_gantt = {
    attach: (context, settings) => {
      $(once('paragraphs_gantt', 'body', context)).each(function () {
        if (settings.gantt) {
          $.each(settings.gantt, function (key, ganttField) {

            // Convert date dự kiến
            if(dataGantt[ganttField.id].data) {
              dataGantt[ganttField.id].data.forEach(function (itemGantt, index) {
                if(itemGantt?.planned_start !== undefined && itemGantt?.planned_end !== undefined) {
                  itemGantt.planned_start = itemGantt.planned_start.concat("T00:00:00");
                  itemGantt.planned_end = itemGantt.planned_end.concat("T00:00:00");
                }
                if(ganttField.permission_edit) {
                  if(itemGantt.hasOwnProperty("owner") || itemGantt.hasOwnProperty("creator")) {
                    let permission_edit = itemGantt.owner.concat(itemGantt.creator);
                    if(permission_edit.length && !permission_edit.includes(settings.user.uid)) {
                      itemGantt.readonly = true
                    }
                  }
                }
              });
            }

            // ajax link action
            let configAjax = {
              url: ganttField.ajax,
              mode: "POST"
            };

            gantt.plugins(plugins);

            markerToday();
            expand();
            setTooltip();
            // undo();
            gantt.config.auto_types = true;
            gantt.config.xml_date = "%Y-%m-%d";
            gantt.config.date_format = "%Y-%m-%d";
            gantt.config.date_grid = settings.date_format;
            gantt.config.duration_unit = "day";
            gantt.config.duration_step = 1;
            gantt.config.order_branch = "marker";
            gantt.config.sort = true;
            gantt.config.fit_tasks = true;
            gantt.config.order_branch_free = true;
            gantt.config.grid_resize = true;
            gantt.config.server_utc = false;
            gantt.config.work_time = false;

            gantt.config.columns.forEach(function (item, key) {
              if (item.hasOwnProperty("name")) {
                if (item.name === "text") {
                  item.min_width = 200;
                }
                if (item.name === "duration") {
                  item.max_width = 100;
                  item.resize = true;
                }
              }
            });

            // set format date tooltips
            gantt.templates.tooltip_date_format = function (date) {
              var formatFunc = gantt.date.date_to_str("%d-%m-%Y");
              return formatFunc(date);
            };

            gantt.$zoomToFit = false;
            gantt.i18n.setLocale(i18n);

            if (Object.values(settings.gantt).length == 1 && !ganttField.split_actual_planned) {

              // button action
              gantt.attachEvent("onLightboxSave", function (id, task, is_new) {
                if (gantt.isReadonly(id)) {
                  gantt.alert(Drupal.t("You cannot edit this task"));
                  return false;
                }
                return true;
              })

              // Custom button, button action
              if (ganttField.show_button_detail) {
                gantt.attachEvent("onGanttReady", function () {
                  gantt.config.buttons_left = ["gantt_save_btn", "details_btn", "gantt_cancel_btn"];
                  gantt.config.buttons_right = ["gantt_delete_btn"];
                });

                gantt.attachEvent("onLightboxButton", function(button_id, node, e, task){
                  if (button_id == "details_btn") {
                    let id = gantt.getState().lightbox;
                    let onTask = gantt.getTask(id);
                    let url = '/paragraphs-gantt/' + onTask.id + '/edit?' + ganttField.current_path;
                    if (onTask.readonly == true) {
                      url = '/paragraphs_item/' + onTask.id + '?' + ganttField.current_path;
                    }
                    let ajaxSettings = {
                      url: url,
                      dialogType: 'modal',
                      dialog: {width: '80%'},
                    };
                    Drupal.ajax(ajaxSettings).execute();
                    gantt.hideLightbox();
                    return false;
                  }
                });
              }

              // Show progress %.
              if (ganttField.has_duration) {
                gantt.templates.progress_text = function (start, end, task) {
                  return "<span style='text-align:left;'>" + Math.round(task.progress * 100) + "% </span>";
                };
                // Change color row.
                gantt.templates.grid_row_class = function (start_date, end_date, item) {
                  if (item.progress == 0) {
                    return "red";
                  }
                  if (item.progress >= 1) {
                    return "green";
                  }
                };
                gantt.templates.task_row_class = function (start_date, end_date, item) {
                  if (item.progress == 0) {
                    return "red";
                  }
                  if (item.progress >= 1) {
                    return "green";
                  }
                };
              }

              // Add column +.
              if (!ganttField.add_task) {
                $.each(gantt.config.columns, function (index, col) {
                  if (col.name === 'add') {
                    delete col.name;
                  }
                });
              }

              // Change mode view readonly.
              if (ganttField.edit_task === false) {
                gantt.config.readonly = true;
              }

              // Switch dialog native drupal.
              if (ganttField.native_dialog) {
                native_dialog(ganttField);
              }

              // Add custom colum custom field.
              if (ganttField.custom_field) {
                let customColumnExist = gantt.config.columns.findIndex((element) => element.name == ganttField.custom_field);
                if (customColumnExist < 0) {
                  gantt.config.columns.splice(gantt.config.columns.length - 1, 0, {
                    name: ganttField.custom_field,
                    label: ganttField.custom_field_name,
                    align: "center",
                    resize: true
                  });
                  gantt.locale.labels['section_' + ganttField.custom_field] = ganttField.custom_field_name;
                }
                let customLightBoxExist = gantt.config.lightbox.sections.findIndex((element) => element.name == ganttField.custom_field);
                if(customLightBoxExist < 0) {
                  gantt.config.lightbox.sections.push(
                    {
                      name: ganttField.custom_field,
                      type: "number",
                      map_to: ganttField.custom_field,
                    }
                  );
                  gantt.form_blocks["number"] = {
                    render: function (sns) {
                      return '<input class="form-control"  type="number" min="0">';
                    },
                    set_value: function (node, value, task) {
                      node.value = value;
                    },
                    get_value: function (node, task) {
                      return node.value || task.options;
                    },
                    focus: function (node) {
                    }
                  };
                }
              }

              // add baseline on row
              // Show 2nd chart for actual date or baseline.
              if (ganttField.has_planned && !ganttField.split_actual_planned) {
                gantt.config.bar_height = 16;
                gantt.config.row_height = 40;
              }
              // Adding baseline display
              if (gantt.hasOwnProperty('addTaskLayer')) {
                gantt.addTaskLayer({
                  renderer: {
                    render: function draw_planned(task) {
                      // add class task when exist base line
                      gantt.templates.task_class  = (start, end, task) => {
                        return "exist_base_line";
                      };
                      // add class link when exist base line
                      gantt.templates.link_class = function(link){
                        return "exist_base_line_link";
                      };
                      // add base line
                      if (task.planned_start && task.planned_end) {
                        var sizes = gantt.getTaskPosition(task, task.planned_start, task.planned_end);
                        var el = document.createElement('div');
                        el.className = 'baseline';
                        el.style.left = sizes.left + 'px';
                        el.style.width = sizes.width + 'px';
                        el.style.top = sizes.top + gantt.config.bar_height + 13 + 'px';
                        return el;
                      }
                      return false;
                    },
                    // Define getRectangle in order to hook layer with the
                    getRectangle: function (task, view) {
                      if (task.planned_start && task.planned_end) {
                        return gantt.getTaskPosition(task, task.planned_start, task.planned_end);
                      }
                      return null;
                    }
                  }
                });

                // Add class Overdue.
                gantt.templates.task_class = function (start, end, task) {
                  if (task.planned_end) {
                    var classes = ['has-baseline'];
                    if (end.getTime() > task.planned_end.getTime()) {
                      classes.push('overdue');
                    }
                    return classes.join(' ');
                  }
                };

                // Calculator Overdue.
                gantt.templates.rightside_text = function (start, end, task) {
                  if (task.planned_end) {
                    if (end.getTime() > task.planned_end.getTime()) {
                      let overdue = Math.ceil(Math.abs((end.getTime() - task.planned_end.getTime()) / (24 * 60 * 60 * 1000)));
                      let text = `<b>${Drupal.t('Overdue')}: ${overdue} ${Drupal.t('days')}</b>`;
                      return text;
                    }
                  }
                };

                gantt.attachEvent("onTaskLoading", function (task) {
                  task.planned_start = gantt.date.parseDate(task.planned_start, "xml_date");
                  task.planned_end = gantt.date.parseDate(task.planned_end, "xml_date");
                  return true;
                });
              }

              // Add workTime and holiday.
              if (ganttField.work_time) {
                gantt.config.work_time = true;
                // add holiday
                if (ganttField.holidays) {
                  let holidays = ganttField.holidays.map(dateString => {
                    let dateParts = dateString.split('-');
                    let year = parseInt(dateParts[0]);
                    let month = parseInt(dateParts[1]) - 1;
                    let day = parseInt(dateParts[2]);
                    gantt.setWorkTime({date: new Date(year, month, day), hours: false});
                    return new Date(year, month, day);
                  });
                }

                gantt.setWorkTime({day: 0, hours: true});
                gantt.setWorkTime({day: 6, hours: true});
                if(ganttField.work_day.length) {
                  const workDays = ganttField.work_day.map(day => {
                    gantt.setWorkTime({day: day, hours: false});
                    return day;
                  });
                }
                gantt.templates.timeline_cell_class = function (task, date) {
                  if (!gantt.isWorkTime({task: task, date: date})) {
                    return "weekend";
                  }
                };
              }

              gantt.ext.zoom.init(zoomConfig);
              gantt.ext.zoom.setLevel("day");


              // Add show colum end Date.
              if (ganttField.show_end) {
                let indexCol = gantt.config.columns.findIndex((item) => item.name == 'start_date');
                gantt.config.columns.splice(indexCol + 1, 0, {
                  name: "end_date",
                  width: 80,
                  label: Drupal.t("End"),
                  align: "center",
                  resize: true,
                  template: function (task) {
                    return gantt.templates.date_grid(task.end_date)
                  }
                });
              }

              // setup tooltip content
              gantt.templates.tooltip_text = function (start, end, task) {
                let html = "<b>" + Drupal.t('Task') + ":</b> " + task.text + "<br/><b>" + Drupal.t("Start") +
                  ":</b> " + gantt.templates.tooltip_date_format(start) +
                  "<br/><b>" + Drupal.t("End") + ":</b> " + gantt.templates.tooltip_date_format(end) +
                  "<br><b>" + Drupal.t("Duration") + ":</b> " + task.duration + Drupal.t(' days') +
                  "<br><b>" + Drupal.t("Creator") + ":</b> " + task.creator_name;
                return html;
              };

              // Set owner for lightbox.
              if (ganttField.owner) {
                owners(ownerGantt[ganttField.id]);
              }

              // Trigger gantt for mono chart.
              gantt.createDataProcessor(configAjax);
              if ($('#' + ganttField.id).length) {
                gantt.init(ganttField.id);
              }
              if (typeof dataGantt != "undefined" && dataGantt[ganttField.id] && dataGantt[ganttField.id].data !== undefined) {
                gantt.parse(dataGantt[ganttField.id]);
              }
            }
            else if (typeof Gantt !== "undefined") {
              // Trigger gantt for multi charts.
              Gantt[key] = Gantt.getGanttInstance();
              Gantt[key].i18n = gantt.i18n;
              Gantt[key].config = gantt.config;
              Gantt[key].plugins(plugins);

              // Custom action button .
              if (ganttField.show_button_detail) {
                Gantt[key].attachEvent("onGanttReady", function () {
                  Gantt[key].config.buttons_left = ["gantt_save_btn", "details_btn", "gantt_cancel_btn"];
                  Gantt[key].config.buttons_right = ["gantt_delete_btn"];
                });

                Gantt[key].attachEvent("onLightboxButton", function (button_id, node, e, task) {
                  if (button_id == "details_btn") {
                    let id = Gantt[key].getState().lightbox;
                    let onTask = Gantt[key].getTask(id);
                    let url = '/paragraphs-gantt/' + onTask.id + '/edit?' + ganttField.current_path;
                    if (onTask.readonly == true) {
                      url = '/paragraphs_item/' + onTask.id + '?' + ganttField.current_path;
                    }
                    let ajaxSettings = {
                      url: url,
                      dialogType: 'modal',
                      dialog: {width: '80%'},
                    };
                    Drupal.ajax(ajaxSettings).execute();
                    Gantt[key].hideLightbox();
                    return false;
                  }
                });
              }

              // Button action.
              Gantt[key].attachEvent("onLightboxSave", function (id, task, is_new) {
                if (Gantt[key].isReadonly(id)) {
                  Gantt[key].alert(Drupal.t("You cannot edit this task"));
                  return false;
                }
                return true;
              })

              // Show progress %.
              if (ganttField.has_duration) {
                Gantt[key].templates.progress_text = function (start, end, task) {
                  return "<span style='text-align:left;'>" + Math.round(task.progress * 100) + "% </span>";
                };
                // Change color row.
                Gantt[key].templates.grid_row_class = function (start_date, end_date, item) {
                  if (item.progress == 0) {
                    return "red";
                  }
                  if (item.progress >= 1) {
                    return "green";
                  }
                };
                Gantt[key].templates.task_row_class = function (start_date, end_date, item) {
                  if (item.progress == 0) {
                    return "red";
                  }
                  if (item.progress >= 1) {
                    return "green";
                  }
                };
              }

              // Add column +.
              if (!ganttField.add_task) {
                $.each(Gantt[key].config.columns, function (index, col) {
                  if (col.name === 'add') {
                    delete col.name;
                  }
                });
              }

              // Change mode view readonly.
              if (ganttField.edit_task === false) {
                Gantt[key].config.readonly = true;
              }

              // Switch dialog native drupal.
              if (ganttField.native_dialog) {
                Gantt[key].attachEvent("onTaskDblClick", function (id, e) {
                  if (id == null) {
                    return false;
                  }
                  task = Gantt[key].getTask(id);
                  if (task['type'] == 'project') {
                    return false;
                  } else {
                    let ajaxSettings = {
                      url: '/paragraphs-gantt/' + id + '/edit?' + ganttField.current_path,
                      dialogType: 'modal',
                      dialog: {width: '80%'},
                    };
                    Drupal.ajax(ajaxSettings).execute();
                    return false;
                  }
                });

                Gantt[key].attachEvent("onBeforeLightbox", function (id) {
                  let task = Gantt[key].getTask(id);
                  if (task == null) {
                    return false;
                  }
                  if (task.$new) {
                    if (ganttField.add_task !== true) {
                      Gantt[key].deleteTask(task.id);
                      return false;
                    }
                    let add_link = ganttField.add_link + '&parent=' + task.parent + '&parent_field=' + ganttField.parent_field;
                    let ajaxSettings = {
                      url: add_link,
                      dialogType: 'modal',
                      dialog: {width: '80%'},
                    };
                    Drupal.ajax(ajaxSettings).execute();
                    Gantt[key].deleteTask(task.id);
                    return false;
                  }
                  return true;
                });
              }

              // Add custom colum custom field.
              if (ganttField.custom_field) {
                let customColumnExist = Gantt[key].config.columns.findIndex((element) => element.name == ganttField.custom_field);
                if (customColumnExist < 0) {
                  Gantt[key].config.columns.splice(Gantt[key].config.columns.length - 1, 0, {
                    name: ganttField.custom_field,
                    label: ganttField.custom_field_name,
                    align: "center",
                    resize: true
                  });
                  Gantt[key].locale.labels['section_' + ganttField.custom_field] = ganttField.custom_field_name;
                }
                let customLightBoxExist = Gantt[key].config.lightbox.sections.findIndex((element) => element.name == ganttField.custom_field);
                if (customLightBoxExist < 0) {
                  Gantt[key].config.lightbox.sections.push(
                    {
                      name: ganttField.custom_field,
                      type: "number",
                      map_to: ganttField.custom_field,
                    }
                  );
                  Gantt[key].form_blocks["number"] = {
                    render: function (sns) {
                      return '<input class="form-control"  type="number" min="0">';
                    },
                    set_value: function (node, value, task) {
                      node.value = value;
                    },
                    get_value: function (node, task) {
                      return node.value || task.options;
                    },
                    focus: function (node) {
                    }
                  };
                }
              }

              // Add workTime and holidays.
              if (ganttField.work_time) {
                Gantt[key].config.work_time = true;

                // add holiday
                if (ganttField.holidays) {
                  let holidays = ganttField.holidays.map(dateString => {
                    let dateParts = dateString.split('-');
                    let year = parseInt(dateParts[0]);
                    let month = parseInt(dateParts[1]) - 1;
                    let day = parseInt(dateParts[2]);
                    Gantt[key].setWorkTime({date: new Date(year, month, day), hours: false});
                    return new Date(year, month, day);
                  });
                }
                if(ganttField.work_day.length) {
                  Gantt[key].setWorkTime({day: 0, hours:true});
                  Gantt[key].setWorkTime({day: 6, hours:true});
                  $.each(ganttField.work_day, function (index, valueItem) {
                    Gantt[key].setWorkTime({day: valueItem, hours:false});
                  });
                }
                Gantt[key].templates.timeline_cell_class = function (task, date) {
                  if (!Gantt[key].isWorkTime({task: task, date: date})) {
                    return "weekend";
                  }
                };
              }

              // Set zoom config for multi charts.
              Gantt[key].ext.zoom.init(zoomConfig);
              Gantt[key].ext.zoom.setLevel("day");

              // Setup tooltip content.
              Gantt[key].templates.tooltip_text = function (start, end, task) {
                let html = "<b>" + Drupal.t('Task') + ":</b> " + task.text + "<br/><b>" + Drupal.t("Start") +
                  ":</b> " + gantt.templates.tooltip_date_format(start) +
                  "<br/><b>" + Drupal.t("End") + ":</b> " + gantt.templates.tooltip_date_format(end) +
                  "<br><b>" + Drupal.t("Duration") + ":</b> " + task.duration + Drupal.t(' days') +
                  "<br><b>" + Drupal.t("Creator") + ":</b> " + task.creator_name;
                return html;
              };

              // Add baseline on row.
              // Show 2nd chart for actual date or baseline.
              if (ganttField.has_planned && !ganttField.split_actual_planned) {
                Gantt[key].config.bar_height = 16;
                Gantt[key].config.row_height = 40;
              }

              // Adding baseline display.
              if (Gantt[key].hasOwnProperty('addTaskLayer')) {
                Gantt[key].addTaskLayer({
                  renderer: {
                    render: function draw_planned(task) {
                      // add class task when exist base line
                      Gantt[key].templates.task_class  = (start, end, task) => {
                        return "exist_base_line";
                      };
                      // add class link when exist base line
                      Gantt[key].templates.link_class = function(link){
                        return "exist_base_line_link";
                      };
                      // add base line
                      if (!isNaN(task.planned_start.getTime()) && !isNaN(task.planned_end.getTime())) {
                        let sizes = Gantt[key].getTaskPosition(task, task.planned_start, task.planned_end);
                        let el = document.createElement('div');
                        el.className = 'baseline';
                        el.style.left = sizes.left + 'px';
                        el.style.width = sizes.width + 'px';
                        el.style.top = sizes.top + Gantt[key].config.bar_height + 13 + 'px';
                        return el;
                      }
                      return false;
                    },
                    // Define getRectangle in order to hook layer with the
                    getRectangle: function (task, view) {
                      if (task.planned_start && task.planned_end) {
                        return Gantt[key].getTaskPosition(task, task.planned_start, task.planned_end);
                      }
                      return null;
                    }
                  }
                });

                // Add class Overdue.
                Gantt[key].templates.task_class = function (start, end, task) {
                  if (task.planned_end) {
                    var classes = ['has-baseline'];
                    if (end.getTime() > task.planned_end.getTime()) {
                      classes.push('overdue');
                    }
                    return classes.join(' ');
                  }
                };

                // Calculator Overdue.
                Gantt[key].templates.rightside_text = function (start, end, task) {
                  if (task.planned_end) {
                    if (end.getTime() > task.planned_end.getTime()) {
                      let overdue = Math.ceil(Math.abs((end.getTime() - task.planned_end.getTime()) / (24 * 60 * 60 * 1000)));
                      let text = `<b>${Drupal.t('Overdue')}: ${overdue} ${Drupal.t('days')}</b>`;
                      return text;
                    }
                  }
                };

                Gantt[key].attachEvent("onTaskLoading", function (task) {
                  if (task.hasOwnProperty("planned_start") && task.hasOwnProperty("planned_end")) {
                    task.planned_start = Gantt[key].date.parseDate(new Date(task.planned_start), "xml_date");
                    task.planned_end = Gantt[key].date.parseDate(new Date(task.planned_end), "xml_date");
                  }
                  return true;
                });
              }

              // Add show colum end Date.
              if (ganttField.show_end) {
                let indexCol = Gantt[key].config.columns.findIndex((item) => item.name == 'start_date');
                Gantt[key].config.columns.splice(indexCol + 1, 0, {
                  name: "end_date",
                  width: 80,
                  label: Drupal.t("End"),
                  align: "center",
                  resize: true,
                  template: function (task) {
                    return Gantt[key].templates.date_grid(task.end_date)
                  }
                });
              }

              // Set owner for dialog.
              if (ganttField.owner) {
                var owner = ownerGantt[ganttField.id];
                Gantt[key].form_blocks["multiselect"] = {
                  render: function (sns) {
                    let height = (sns.height || "23") + "px";
                    let html = "<div class='gantt_cal_ltext gantt_cal_chosen gantt_cal_multiselect' style='height:" + height + ";width:100%'><select data-placeholder='...' class='chosen-select' multiple>";
                    if (sns.options) {
                      for (var i = 0; i < sns.options.length; i++) {
                        if(sns.unassigned_value !== undefined && sns.options[i].key == sns.unassigned_value){
                          continue;
                        }
                        html += "<option value='" + sns.options[i].key + "'>" + sns.options[i].label + "</option>";
                      }
                    }
                    html += "</select></div>";
                    return html;
                  },

                  set_value: function (node, value, ev, sns) {
                    node.style.overflow = "visible";
                    node.parentNode.style.overflow = "visible";
                    node.style.display = "inline-block";
                    let select = $(node.firstChild);

                    if (value) {
                      value = (value + "").split(",");
                      select.val(value);
                    }
                    else {
                      select.val([]);
                    }

                    select.chosen();
                    if (sns.onchange) {
                      select.change(function () {
                        sns.onchange.call(this);
                      })
                    }
                    select.trigger('chosen:updated');
                    select.trigger("change");
                  },

                  get_value: function (node, ev) {
                    return $(node.firstChild).val();
                  },

                  focus: function (node) {
                    $(node.firstChild).focus();
                  }
                };

                Gantt[key].locale.labels.column_owner =
                  Gantt[key].locale.labels.section_owner = Drupal.t("Owner");
                Gantt[key].config.resource_property = "staff";
                Gantt[key].serverList("staff", owner);

                Gantt[key].attachEvent("onTaskCreated", function (task) {
                  task.staff = [];
                  task.calendar_id = null;
                  return true;
                });
                // Add column owner.
                let colOwnerExists = Gantt[key].config.columns.findIndex((element) => element.name == "owner");
                if (colOwnerExists < 0) {
                  Gantt[key].config.columns.unshift({
                    name: "owner", width: 80, align: "center", template: (task) => {
                      if (task.type == Gantt[key].config.types.project) {
                        return "";
                      }
                      var result = "";
                      let owners = task.owner;
                      if (!owners || !owners.length) {
                        return "Unassigned";
                      }
                      if (owners.length == 1) {
                        return byId(Gantt[key].serverList('staff'), task.owner[0]);
                      }
                      owners.forEach((ownerId) => {
                        let owner = byId(Gantt[key].serverList('staff'), ownerId);
                        if (!owner) {
                          return;
                        }
                        result += "<div class='owner-label' title='" + owner + "'>" + owner.charAt(0).toUpperCase() + "</div>";
                      });
                      return result;
                    }, resize: true
                  });
                }

                // Add lightBox owner.
                let lightBoxOwnerExists = Gantt[key].config.lightbox.sections.findIndex((element) => element.name == "owner");
                if (lightBoxOwnerExists < 0) {
                  Gantt[key].config.lightbox.sections.push(
                    {
                      name: "owner",
                      height: 60,
                      type: "multiselect",
                      options: Gantt[key].serverList("staff"),
                      map_to: "owner"
                    }
                  );
                }

                Gantt[key].templates.grid_row_class =
                  Gantt[key].templates.task_row_class =
                    Gantt[key].templates.task_class = function (start, end, task) {
                      let css = [];
                      if (task.$virtual || task.type == Gantt[key].config.types.project) {
                        css.push("summary-bar");
                      }

                      if (task.owner) {
                        css.push("gantt_resource_task gantt_resource_" + task.owner);
                      }

                      return css.join(" ");
                    };

                Gantt[key].attachEvent("onDataRender", function () {
                  let styleId = "dynamicGanttStyles";
                  let element = document.getElementById(styleId);
                  if (!element) {
                    element = document.createElement("style");
                    element.id = styleId;
                    document.querySelector("head").appendChild(element);
                  }
                  let html = [];
                  let resources = Gantt[key].serverList("staff");

                  resources.forEach(function (r) {
                    html.push(".gantt_task_line.gantt_resource_" + r.key + "{" +
                      "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
                    html.push(".gantt_row.gantt_resource_" + r.key + " .gantt_cell:nth-child(1) .gantt_tree_content{" +
                      "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
                  });
                  element.innerHTML = html.join("");
                });
              }

              // Trigger gantt for mono chart.
              Gantt[key].createDataProcessor(configAjax);
              if ($('#' + ganttField.id).length) {
                Gantt[key].init(ganttField.id);
              }
              if ($('#' + ganttField.id).length && dataGantt[ganttField.id].data !== undefined) {
                Gantt[key].parse(dataGantt[ganttField.id]);
              }

              if (ganttField.split_actual_planned && typeof dataPlanned !== 'undefined' && dataPlanned[ganttField.id] !== undefined) {
                // Show 2nd chart for planed date.
                Gantt[key + 'plan'] = Gantt.getGanttInstance();
                Gantt[key + 'plan'].config = gantt.config;
                Gantt[key + 'plan'].createDataProcessor({
                  url: ganttField.ajax + '?isPlanned=1',
                  mode: "POST"
                });
                Gantt[key + 'plan'].init('planned-' + ganttField.id);
                Gantt[key + 'plan'].parse(dataPlanned[ganttField.id]);

                Gantt[key + 'plan'].plugins({
                  tooltip: true,
                });
              }
            }
          });
        }
      });
    },
  };
})(jQuery, Drupal, window.Cookies);
