<?php

namespace Drupal\paragraphs_gantt\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\paragraphs\Entity\Paragraph;
use Drupal\paragraphs\ParagraphsTypeInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class definition for ComponentFormController.
 */
class ParagraphsGanttController extends ControllerBase {

  /**
   * Query POST.
   *
   * @var mixed
   */
  private $post;

  /**
   * Add new / Edit / Delete Paragraphs.
   */
  public function ajax(Request $request, ParagraphsTypeInterface $paragraph_type, $view_mode, $entity_type, $entity_field, $entity_id) {

    $entity = $this->entityTypeManager()
      ->getStorage($entity_type)
      ->load($entity_id);

    $bundle = $entity->bundle();
    $loadViewDisplay = $this->entityTypeManager()
      ->getStorage('entity_view_display')
      ->load($entity_type . '.' . $bundle . '.' . $view_mode);
    $fieldSetting = $loadViewDisplay->getRenderer($entity_field)->getSettings();
    $action = $request->request->get("!nativeeditor_status") ?? FALSE;
    $output = [];
    if (!$action) {
      return new JsonResponse($output);
    }
    $this->post = $request->request->all();
    $mode = $request->query->get('gantt_mode');
    $isPlanned = $request->query->get('isPlanned');
    if ($mode == 'tasks') {
      $output = $this->actionTask($action, $entity, $paragraph_type, $entity_field, $fieldSetting, $isPlanned);
    }
    if ($mode == 'links') {
      $output = $this->actionLinks($action, $entity, $paragraph_type, $entity_field, $fieldSetting);
    }
    return new JsonResponse($output);
  }

  /**
   * Import file MPP.
   *
   * {@inheritdoc}
   */
  public function import(Request $request, ParagraphsTypeInterface $paragraph_type, $view_mode, $entity_type, $entity_field, $entity_id) {
    $action = 'inserted';
    $output = ['action' => $action];
    $entity = $this->entityTypeManager()
      ->getStorage($entity_type)
      ->load($entity_id);
    $mapping = [];
    $bundle = $entity->bundle();
    $loadViewDisplay = $this->entityTypeManager()
      ->getStorage('entity_view_display')
      ->load($entity_type . '.' . $bundle . '.' . $view_mode);
    $fieldSetting = $loadViewDisplay->getRenderer($entity_field)->getSettings();
    if (!empty($request->get("data"))) {
      $rawData = $request->get("data");
      $rawLinks = $request->get("links");
      $this->post = [];
      foreach ($rawData as $data) {
        $ganttData[$data['id']] =
        $this->post = [
          'text' => $data['text'],
          'duration' => $data['duration'],
          'progress' => $data['progress'],
          'open' => $data['open'],
          'parent' => $data['parent'],
          'type' => $data['type'] ?? 'task',
          'start_date' => $data['$raw']["Start"],
          'end_date' => $data['$raw']["Finish"],
        ];
        if (!empty($mapping[$data['parent']])) {
          $this->post['parent'] = $mapping[$data['parent']];
        }
        $output = $this->actionTask($action, $entity, $paragraph_type, $entity_field, $fieldSetting);
        $mapping[$data['id']] = $output['tid'];
      }

      foreach ($rawLinks as $data) {
        $this->post = [
          'source' => $mapping[$data['source']],
          'target' => $mapping[$data['target']],
          'type' => $data['type'],
        ];
        $output = $this->actionLinks($action, $entity, $paragraph_type, $entity_field, $fieldSetting);
      }
    }

    return new JsonResponse($output);
  }

  /**
   * Convert date return with format Y-m-d.
   */
  protected function formatDate($date, $type = 'date', $format = 'Y-m-d H:i:s') {
    if (empty($date)) {
      return '';
    }
    $dateFormatStorage = $this->entityTypeManager->getStorage('date_format');
    $drupalShortFormat = $dateFormatStorage->load('short')->getPattern();
    if ($type == 'date') {
      $format = current(explode(' ', $format));
      $drupalShortFormat = current(explode(' ', $drupalShortFormat));
    }
    $convertDateTime = \DateTime::createFromFormat($drupalShortFormat, $date);
    if (!$convertDateTime) {
      $date = str_replace('/', '-', $date);
      $timestamp = strtotime($date);
      return date($format, $timestamp);
    }
    $dateFormated = $convertDateTime->format($format);
    // Take only date remove hours.
    $date = explode(' ', str_replace('T', ' ', $dateFormated));
    return current($date);
  }

  /**
   * Process Task.
   */
  private function actionTask($action, $entity, $paragraph_type, $entity_field, $field_settings, $isPlanned = FALSE) {
    $output = ['action' => $action];
    if (!empty($this->post['isImport'])) {
      return new JsonResponse($output);
    }
    if ($action == 'inserted') {
      $paragraph = Paragraph::create(['type' => $paragraph_type->id()]);
    }
    else {
      $paragraph = Paragraph::load($this->post["id"]);
    }

    if ($action == 'deleted') {
      $getField = $entity->get($entity_field);
      $array_of_referenced_items = $getField->getValue();
      $index_to_remove = array_search($this->post['id'], array_column($array_of_referenced_items, 'target_id'));
      if ($index_to_remove >= 0) {
        $getField->removeItem($index_to_remove);
        $entity->save();
        $paragraph->delete();
        $storage = $this->entityTypeManager()
          ->getStorage($entity->getEntityTypeId());
        $storage->resetCache([$entity->id()]);
      }
      return new JsonResponse($output);
    }
    if ($action == 'order') {
      if (is_numeric($this->post["target"])) {
        $targetTaskId = $this->post["target"];
        $nextTask = FALSE;
      }
      else {
        $target = explode(':', $this->post["target"]);
        $targetTaskId = end($target);
        $nextTask = TRUE;
      }
      // @todo Fix update order with delta in paragraphs.
      if ($nextTask && $targetTaskId) {
      }
    }

    $fieldDefinitions = $paragraph->getFieldDefinitions();
    $fieldType = $dateType = 'date';
    if (!empty($fieldDefinitions[$field_settings['start_date']])) {
      $dateSettings = $fieldDefinitions[$field_settings['start_date']]->getSettings();
      $dateType = $dateSettings["datetime_type"];
      $fieldType = $fieldDefinitions[$field_settings['start_date']]->getType();
    }
    $dateFormat = $fieldType == 'daterange' ? 'Y-m-d\TH:i:s' : 'Y-m-d H:i:s';
    // Set start date.
    $actual_date = [];
    if (!empty($this->post['start_date'])) {
      $start_date = $this->formatDate($this->post['start_date'], $dateType, $dateFormat);
      $end_date = $this->formatDate($this->post['end_date'], $dateType, $dateFormat);
      // Remove empty hour.
      $start_date = str_replace(['T00:00:00', ' 00:00:00'], '', $start_date);
      $end_date = str_replace(['T00:00:00', ' 00:00:00'], '', $end_date);
      if ($fieldType == 'daterange' && !empty($start_date)) {
        $actual_date = [
          'value' => $start_date,
          'end_value' => $end_date,
        ];
      }
    }
    // Set plan date.
    $planned_date = [
      'value' => NULL,
      'end_value' => NULL,
    ];
    if (!empty($this->post['planned_start'])) {
      $planned_start = $this->formatDate($this->post['planned_start'], $dateType, $dateFormat);
      $planned_end = $this->formatDate($this->post['planned_end'], $dateType, $dateFormat);
      // Remove empty hour.
      $planned_start = str_replace(['T00:00:00', ' 00:00:00'], '', $planned_start);
      $planned_end = str_replace(['T00:00:00', ' 00:00:00'], '', $planned_end);
      if ($fieldType == 'daterange' && !empty($planned_start)) {
        $planned_date = [
          'value' => $planned_start,
          'end_value' => $planned_end,
        ];
      }
    }
    // Set actual date and plan date.
    if (!empty($actual_date)) {
      if (empty($isPlanned)) {
        $paragraph->set($field_settings['start_date'], $actual_date);
        $paragraph->set($field_settings['duration'], $this->post['duration']);
      }
      else {
        $paragraph->set($field_settings['planned_date'], $actual_date);
        $paragraph->set($field_settings['planned_duration'], $this->post['duration']);
      }
      if (!empty($planned_date) && !empty($field_settings['planned_date'])) {
        $paragraph->set($field_settings['planned_date'], $planned_date);
        $planned_duration = round(abs(strtotime($planned_start) - strtotime($planned_end)) / 86400);
        $paragraph->set($field_settings['planned_duration'], $planned_duration);
      }
    }
    // Set text.
    if (!empty($field_settings['text'])) {
      $paragraph->set($field_settings['text'], $this->post['text']);
    }
    // Set type.
    if (!empty($field_settings["type"]) && !empty($this->post["type"])) {
      $paragraph->set($field_settings['type'], $this->post["type"]);
    }
    // Set progress.
    if (!empty($field_settings['progress']) && !empty($this->post['progress'])) {
      $paragraph->set($field_settings['progress'], $this->post['progress']);
    }
    // Set parent.
    $parent = !empty($this->post['parent']) ? $this->post['parent'] : 0;
    if (!empty($field_settings['parent'])) {
      $paragraph->set($field_settings['parent'], $parent);
    }
    // Set open.
    $open = !empty($this->post['open']) ? $this->post['open'] : 0;
    if (!empty($field_settings['open'])) {
      $paragraph->set($field_settings['open'], $open);
    }
    // Set owner.
    if (!empty($field_settings["owner"])) {
      $owner = [];
      $this->post['owner'] = json_decode($this->post['owner']);
      if (is_array($this->post['owner'])) {
        foreach ($this->post['owner'] as $owner_id) {
          $owner[] = ['target_id' => $owner_id];
        }
      }
      else {
        $owner = [['target_id' => $this->post['owner']]];
      }
      $paragraph->set($field_settings['owner'], $owner);
    }
    // Set custom field.
    if (!empty($field_settings["custom_field"]) && !empty($this->post[$field_settings["custom_field"]])) {
      $paragraph->set($field_settings['custom_field'], $this->post[$field_settings["custom_field"]]);
    }
    // Set creator.
    if (!empty($field_settings["creator"]) && $action == 'inserted') {
      $paragraph->set($field_settings["creator"], $this->currentUser()->id());
    }
    if ($action == 'inserted') {
      $paragraph->isNew();
    }
    $paragraph->save();

    if ($action == 'inserted') {
      $output['tid'] = $paragraph->id();
      $fieldValue = $entity->get($entity_field)->getValue();
      $fieldValue[] = [
        'target_id' => $paragraph->id(),
        'target_revision_id' => $paragraph->getRevisionId(),
      ];
      $entity->set($entity_field, $fieldValue);
    }
    $entity->save();
    $storage = $this->entityTypeManager()->getStorage($entity->getEntityTypeId());
    $storage->resetCache([$entity->id()]);

    return $output;
  }

  /**
   * {@inheritDoc}
   */
  private function actionLinks(string $action, EntityInterface $entity, ParagraphsTypeInterface $paragraph_type, string $entity_field, array $field_settings) {
    $output = ['action' => $action];
    $source = !empty($this->post['source']) ? $this->post['source'] : FALSE;
    $target = !empty($this->post['target']) ? $this->post['target'] : FALSE;
    $type = !empty($this->post['type']) ? $this->post['type'] : FALSE;
    if ($action == 'deleted') {
      [$source, $target] = explode('-', $this->post['id']);
    }
    $paragraph = Paragraph::load($source);
    $linkValues = $paragraph->get($field_settings['links'])->getValue();

    if ($action == 'inserted' || !empty($this->post['isImport'])) {
      $linkValues[] = ['first' => $target, 'second' => $type];
    }
    if ($action == 'deleted') {
      foreach ($linkValues as $delta => $linkValue) {
        if ($linkValue['first'] == $target) {
          unset($linkValues[$delta]);
        }
      }
    }
    $paragraph->set($field_settings['links'], $linkValues);

    $paragraph->save();
    $entity = $paragraph->getParentEntity();
    $storage = $this->entityTypeManager()->getStorage($entity->getEntityTypeId());
    $storage->resetCache([$entity->id()]);
    return new JsonResponse($output);
  }

}
